@@ -212,11 +212,13 @@ attempt_to_merge(int rootfd, struct pkg_config_file *rcf, struct pkg *local,
 		pkg_debug(2, "Ancient vanilla and deployed conf are the same size testing checksum");
 		localsum = pkg_checksum_data(localconf, sz,
 		    PKG_HASH_TYPE_SHA256_HEX);
-		if (localsum && STREQ(localsum, lf->sum)) {
+		if (localsum != NULL && lf->sum != NULL && STREQ(localsum, lf->sum)) {
 			pkg_debug(2, "Checksum are the same %jd", (intmax_t)strlen(localconf));
 			free(localsum);
 			goto ret;
 		}
+		if (lf->sum == NULL)
+			pkg_emit_error("3way merge: no checksum for the original local file");
 		free(localsum);
 		pkg_debug(2, "Checksum are different %jd", (intmax_t)strlen(localconf));
 	}
@@ -246,56 +248,6 @@ ret:
 	free(localconf);
 }
 
-static uid_t
-get_uid_from_archive(struct archive_entry *ae)
-{
-	static char user_buffer[1024];
-	const char *user;
-	static struct passwd pwent;
-	struct passwd *result;
-	int err;
-
-	user = archive_entry_uname(ae);
-	if (pwent.pw_name != NULL && STREQ(user, pwent.pw_name))
-		goto out;
-	pwent.pw_name = NULL;
-	err = getpwnam_r(user, &pwent, user_buffer, sizeof(user_buffer),
-	    &result);
-	if (err != 0) {
-		pkg_emit_errno("getpwnam_r", user );
-		return (0);
-	}
-	if (result == NULL)
-		return (0);
-out:
-	return (pwent.pw_uid);
-}
-
-static gid_t
-get_gid_from_archive(struct archive_entry *ae)
-{
-	static char group_buffer[1024];
-	static struct group grent;
-	struct group *result;
-	const char *group;
-	int err;
-
-	group = archive_entry_gname(ae);
-	if (grent.gr_name != NULL && STREQ(group, grent.gr_name))
-		goto out;
-	grent.gr_name = NULL;
-	err = getgrnam_r(group, &grent, group_buffer, sizeof(group_buffer),
-	    &result);
-	if (err != 0) {
-		pkg_emit_errno("getgrnam_r", group );
-		return (0);
-	}
-	if (result == NULL)
-		return (0);
-out:
-	return (grent.gr_gid);
-}
-
 static int
 set_chflags(int fd, const char *path, u_long fflags)
 {
@@ -374,15 +326,15 @@ fill_timespec_buf(const struct stat *aest, struct timespec tspec[2])
 {
 #ifdef HAVE_STRUCT_STAT_ST_MTIM
 	tspec[0].tv_sec = aest->st_atim.tv_sec;
-	tspec[0].tv_nsec = aest->st_atim.tv_nsec;
+	tspec[0].tv_nsec = 0;
 	tspec[1].tv_sec = aest->st_mtim.tv_sec;
-	tspec[1].tv_nsec = aest->st_mtim.tv_nsec;
+	tspec[1].tv_nsec = 0;
 #else
 # if defined(_DARWIN_C_SOURCE) || defined(__APPLE__)
 	tspec[0].tv_sec = aest->st_atimespec.tv_sec;
-	tspec[0].tv_nsec = aest->st_atimespec.tv_nsec;
+	tspec[0].tv_nsec = 0;
 	tspec[1].tv_sec = aest->st_mtimespec.tv_sec;
-	tspec[1].tv_nsec = aest->st_mtimespec.tv_nsec;
+	tspec[1].tv_nsec = 0;
 # else
 	/* Portable unix version */
 	tspec[0].tv_sec = aest->st_atime;
@@ -432,12 +384,13 @@ close_tempdir(struct tempdir *t)
 }
 
 static int
-create_dir(struct pkg_add_context *context, struct pkg_dir *d, tempdirs_t *tempdirs)
+create_dir(struct pkg_add_context *context, struct pkg_dir *d,
+    tempdirs_t *tempdirs)
 {
 	struct stat st;
-	struct tempdir *tmpdir = NULL;
-	int fd;
+	struct tempdir *tmpdir;
 	const char *path;
+	int error, fd;
 
 	tmpdir = get_tempdir(context, d->path, tempdirs);
 	if (tmpdir == NULL) {
@@ -448,25 +401,39 @@ create_dir(struct pkg_add_context *context, struct pkg_dir *d, tempdirs_t *tempd
 		path = d->path + tmpdir->len;
 	}
 
-	if (mkdirat(fd, RELATIVE_PATH(path), 0755) == -1)
+	if (mkdirat(fd, RELATIVE_PATH(path), 0755) == -1) {
 		if (!mkdirat_p(fd, RELATIVE_PATH(path))) {
 			close_tempdir(tmpdir);
 			return (EPKG_FATAL);
 		}
+	}
+
 	if (fstatat(fd, RELATIVE_PATH(path), &st, 0) == -1) {
 		if (errno != ENOENT) {
 			close_tempdir(tmpdir);
-			pkg_fatal_errno("Fail to stat directory %s", path);
+			pkg_fatal_errno("Fail to stat directory %s", d->path);
 		}
-		if (fstatat(fd, RELATIVE_PATH(path), &st, AT_SYMLINK_NOFOLLOW) == 0) {
-			unlinkat(fd, RELATIVE_PATH(path), 0);
+		/* path is a dangling symlink. */
+		error = unlinkat(fd, RELATIVE_PATH(path), 0);
+		if (error != 0) {
+			close_tempdir(tmpdir);
+			pkg_fatal_errno("Fail to unlink dangling symlink %s",
+			    d->path);
 		}
 		if (mkdirat(fd, RELATIVE_PATH(path), 0755) == -1) {
 			if (tmpdir != NULL) {
 				close_tempdir(tmpdir);
-				pkg_fatal_errno("Fail to create directory '%s/%s'", tmpdir->temp, path);
+				pkg_fatal_errno(
+				    "Fail to create directory %s/%s",
+				    tmpdir->temp, path);
+			} else {
+				pkg_fatal_errno("Fail to create directory %s",
+				    path);
 			}
-			pkg_fatal_errno("Fail to create directory %s", path);
+		}
+		if (fstatat(fd, RELATIVE_PATH(path), &st, 0) == -1) {
+			close_tempdir(tmpdir);
+			pkg_fatal_errno("Fail to stat directory %s", d->path);
 		}
 	}
 
@@ -496,8 +463,10 @@ do_extract_dir(struct pkg_add_context* context, struct archive *a __unused, stru
 	}
 	aest = archive_entry_stat(ae);
 	d->perm = aest->st_mode;
-	d->uid = get_uid_from_archive(ae);
-	d->gid = get_gid_from_archive(ae);
+	d->uid = get_uid_from_uname(archive_entry_uname(ae));
+	d->gid = get_gid_from_gname(archive_entry_gname(ae));
+	strlcpy(d->uname, archive_entry_uname(ae), sizeof(d->uname));
+	strlcpy(d->gname, archive_entry_gname(ae), sizeof(d->gname));
 	fill_timespec_buf(aest, d->time);
 	archive_entry_fflags(ae, &d->fflags, &clear);
 
@@ -572,8 +541,9 @@ retry:
 
 /* In case of a symlink create it directly with a random name */
 static int
-do_extract_symlink(struct pkg_add_context *context, struct archive *a __unused, struct archive_entry *ae,
-    const char *path, struct pkg *local __unused, tempdirs_t *tempdirs)
+do_extract_symlink(struct pkg_add_context *context, struct archive *a __unused,
+    struct archive_entry *ae, const char *path, struct pkg *local __unused,
+    tempdirs_t *tempdirs)
 {
 	struct pkg_file *f;
 	const struct stat *aest;
@@ -587,8 +557,10 @@ do_extract_symlink(struct pkg_add_context *context, struct archive *a __unused,
 
 	aest = archive_entry_stat(ae);
 	archive_entry_fflags(ae, &f->fflags, &clear);
-	f->uid = get_uid_from_archive(ae);
-	f->gid = get_gid_from_archive(ae);
+	f->uid = get_uid_from_uname(archive_entry_uname(ae));
+	f->gid = get_gid_from_gname(archive_entry_gname(ae));
+	strlcpy(f->uname, archive_entry_uname(ae), sizeof(f->uname));
+	strlcpy(f->gname, archive_entry_gname(ae), sizeof(f->gname));
 	f->perm = aest->st_mode;
 	fill_timespec_buf(aest, f->time);
 	archive_entry_fflags(ae, &f->fflags, &clear);
@@ -833,8 +805,10 @@ do_extract_regfile(struct pkg_add_context *context, struct archive *a, struct ar
 	aest = archive_entry_stat(ae);
 	archive_entry_fflags(ae, &f->fflags, &clear);
 	f->perm = aest->st_mode;
-	f->uid = get_uid_from_archive(ae);
-	f->gid = get_gid_from_archive(ae);
+	f->uid = get_uid_from_uname(archive_entry_uname(ae));
+	f->gid = get_gid_from_gname(archive_entry_gname(ae));
+	strlcpy(f->uname, archive_entry_uname(ae), sizeof(f->uname));
+	strlcpy(f->gname, archive_entry_gname(ae), sizeof(f->gname));
 	fill_timespec_buf(aest, f->time);
 	archive_entry_fflags(ae, &f->fflags, &clear);
 
@@ -1866,10 +1840,15 @@ struct tempdir *
 open_tempdir(struct pkg_add_context *context, const char *path)
 {
 	struct stat st;
+	struct pkg *localpkg;
 	char walk[MAXPATHLEN];
 	char *dir;
 	size_t cnt = 0, len;
 	struct tempdir *t;
+	int rootfd;
+
+	rootfd = context->rootfd;
+	localpkg = context->localpkg;
 
 	strlcpy(walk, path, sizeof(walk));
 	while ((dir = strrchr(walk, '/')) != NULL) {
@@ -1880,15 +1859,29 @@ open_tempdir(struct pkg_add_context *context, const char *path)
 		if (len == 0 && cnt == 1)
 			break;
 		if (len > 0) {
-			int flag = AT_SYMLINK_NOFOLLOW;
-			if (context->localpkg == NULL)
-				flag = 0;
-			if (fstatat(context->rootfd, RELATIVE_PATH(walk), &st, flag) == -1)
+			int flag;
+
+			flag = localpkg == NULL ? 0 : AT_SYMLINK_NOFOLLOW;
+			if (fstatat(rootfd, RELATIVE_PATH(walk), &st,
+			    flag) == -1) {
+				/*
+				 * A hack to ensure that intermediate
+				 * directories not registered with a package are
+				 * still logged in the metalog.  This is not
+				 * really the right place, but to implement this
+				 * properly we need to clean up uses of
+				 * try_mkdir().
+				 */
+				metalog_add(PKG_METALOG_DIR,
+				    RELATIVE_PATH(walk),
+				    "root", "wheel", 0755, 0, NULL);
 				continue;
-			if (S_ISLNK(st.st_mode) && context->localpkg != NULL && pkghash_get(context->localpkg->filehash, walk) == NULL) {
-				if (fstatat(context->rootfd, RELATIVE_PATH(walk), &st, 0) == -1)
-					continue;
 			}
+			if (S_ISLNK(st.st_mode) &&
+			    localpkg != NULL &&
+			    pkghash_get(localpkg->filehash, walk) == NULL &&
+			    fstatat(rootfd, RELATIVE_PATH(walk), &st, 0) == -1)
+				continue;
 			if (S_ISDIR(st.st_mode) && cnt == 1)
 				break;
 			if (!S_ISDIR(st.st_mode))
@@ -1897,7 +1890,7 @@ open_tempdir(struct pkg_add_context *context, const char *path)
 		*dir = '/';
 		t = xcalloc(1, sizeof(*t));
 		hidden_tempfile(t->temp, sizeof(t->temp), walk);
-		if (mkdirat(context->rootfd, RELATIVE_PATH(t->temp), 0755) == -1) {
+		if (mkdirat(rootfd, RELATIVE_PATH(t->temp), 0755) == -1) {
 			pkg_errno("Fail to create temporary directory: %s", t->temp);
 			free(t);
 			return (NULL);
@@ -1905,7 +1898,7 @@ open_tempdir(struct pkg_add_context *context, const char *path)
 
 		strlcpy(t->name, walk, sizeof(t->name));
 		t->len = strlen(t->name);
-		t->fd = openat(context->rootfd, RELATIVE_PATH(t->temp), O_DIRECTORY);
+		t->fd = openat(rootfd, RELATIVE_PATH(t->temp), O_DIRECTORY);
 		if (t->fd == -1) {
 			pkg_errno("Fail to open directory %s", t->temp);
 			free(t);
