@@ -70,7 +70,8 @@
 
 extern struct pkg_ctx ctx;
 
-static int pkg_jobs_installed_local_pkg(struct pkg_jobs *j, struct pkg *pkg);
+static int _pkg_is_installed(struct pkg_jobs *j, const char *pattern,
+    match_t match);
 static int pkg_jobs_find_upgrade(struct pkg_jobs *j, const char *pattern, match_t m);
 static int pkg_jobs_fetch(struct pkg_jobs *j);
 static bool new_pkg_version(struct pkg_jobs *j);
@@ -90,14 +91,7 @@ pkg_jobs_new(struct pkg_jobs **j, pkg_jobs_t t, struct pkgdb *db)
 	assert(db != NULL);
 
 	*j = xcalloc(1, sizeof(struct pkg_jobs));
-
 	(*j)->universe = pkg_jobs_universe_new(*j);
-
-	if ((*j)->universe == NULL) {
-		free(*j);
-		return (EPKG_FATAL);
-	}
-
 	(*j)->db = db;
 	(*j)->type = t;
 	(*j)->solved = false;
@@ -401,7 +395,7 @@ pkg_jobs_add_req(struct pkg_jobs *j, struct pkg *pkg)
 
 	dbg(4, "add package %s-%s to the request", pkg->name,
 			pkg->version);
-	rc = pkg_jobs_universe_add_pkg(j->universe, pkg, false, &un);
+	rc = pkg_jobs_universe_add_pkg(j->universe, pkg, &un);
 
 	if (rc == EPKG_END) {
 		/*
@@ -918,7 +912,8 @@ pkg_jobs_find_upgrade(struct pkg_jobs *j, const char *pattern, match_t m)
 	 */
 	checklocal = j->type == PKG_JOBS_UPGRADE && m != MATCH_EXACT && m != MATCH_ALL;
 	while (it != NULL && pkgdb_it_next(it, &p, flags) == EPKG_OK) {
-		if (checklocal && pkg_jobs_installed_local_pkg(j, p) != EPKG_OK)
+		if (checklocal &&
+		    _pkg_is_installed(j, p->name, MATCH_INTERNAL) != EPKG_OK)
 			continue;
 		if (pattern != NULL && *pattern != '@') {
 			with_version = strcmp(p->name, pattern);
@@ -930,9 +925,6 @@ pkg_jobs_find_upgrade(struct pkg_jobs *j, const char *pattern, match_t m)
 			break;
 		} else if (rc == EPKG_OK)
 			found = true;
-
-		pkg_free(p);
-		p = NULL;
 	}
 
 	pkgdb_it_free(it);
@@ -958,7 +950,7 @@ pkg_jobs_find_upgrade(struct pkg_jobs *j, const char *pattern, match_t m)
 
 		dbg(2, "non-automatic package with pattern %s has not been found in "
 				"remote repo", pattern);
-		rc = pkg_jobs_universe_add_pkg(j->universe, p, false, &unit);
+		rc = pkg_jobs_universe_add_pkg(j->universe, p, &unit);
 		if (rc == EPKG_OK) {
 			rc = pkg_jobs_guess_upgrade_candidate(j, pattern);
 		}
@@ -967,37 +959,30 @@ pkg_jobs_find_upgrade(struct pkg_jobs *j, const char *pattern, match_t m)
 	return (rc);
 }
 
+/*
+ * Return EPKG_OK if a package with a name matching the pattern is installed
+ * locally.
+ */
 static int
-pkg_jobs_check_local_pkg(struct pkg_jobs *j, struct job_pattern *jp)
+_pkg_is_installed(struct pkg_jobs *j, const char *pattern, match_t match)
 {
 	struct pkgdb_it *it;
 	struct pkg *pkg = NULL;
-	int rc = EPKG_OK;
+	int ret;
 
-	it = pkgdb_query(j->db, jp->pattern, jp->match);
+	it = pkgdb_query(j->db, pattern, match);
 	if (it != NULL) {
-		if (pkgdb_it_next(it, &pkg, PKG_LOAD_BASIC|PKG_LOAD_ANNOTATIONS) != EPKG_OK)
-			rc = EPKG_FATAL;
-		else
+		ret = pkgdb_it_next(it, &pkg,
+		    PKG_LOAD_BASIC | PKG_LOAD_ANNOTATIONS);
+		if (ret == EPKG_OK)
 			pkg_free(pkg);
-
+		else if (ret == EPKG_END)
+			ret = EPKG_NOTINSTALLED;
 		pkgdb_it_free(it);
+	} else {
+		ret = EPKG_FATAL;
 	}
-	else {
-		rc = EPKG_FATAL;
-	}
-
-	return (rc);
-}
-
-static int
-pkg_jobs_installed_local_pkg(struct pkg_jobs *j, struct pkg *pkg)
-{
-	struct job_pattern jfp;
-
-	jfp.match = MATCH_INTERNAL;
-	jfp.pattern = pkg->name;
-	return (pkg_jobs_check_local_pkg(j, &jfp));
+	return (ret);
 }
 
 static int
@@ -1008,42 +993,42 @@ pkg_jobs_find_remote_pattern(struct pkg_jobs *j, struct job_pattern *jp)
 	struct pkg_job_request *req;
 
 	if (!(jp->flags & PKG_PATTERN_FLAG_FILE)) {
-		if (j->type == PKG_JOBS_UPGRADE && (jp->match == MATCH_INTERNAL || jp->match == MATCH_EXACT)) {
+		if (j->type == PKG_JOBS_UPGRADE &&
+		    (jp->match == MATCH_INTERNAL || jp->match == MATCH_EXACT)) {
 			/*
 			 * For upgrade patterns we must ensure that a local package is
 			 * installed as well.  This only works if we're operating on an
 			 * exact match, as we otherwise don't know exactly what packages
 			 * are in store for us.
 			 */
-			if (pkg_jobs_check_local_pkg(j, jp) != EPKG_OK) {
-				pkg_emit_error("%s is not installed, therefore upgrade is impossible",
-						jp->pattern);
-				return (EPKG_NOTINSTALLED);
+			rc = _pkg_is_installed(j, jp->pattern, jp->match);
+			if (rc != EPKG_OK) {
+				pkg_emit_error(
+				    "%s is not installed, therefore upgrade is impossible",
+				    jp->pattern);
+				return (rc);
 			}
 		}
 		rc = pkg_jobs_find_upgrade(j, jp->pattern, jp->match);
-	}
-	else {
-		if (pkg_open(&pkg, jp->path, PKG_OPEN_MANIFEST_ONLY) != EPKG_OK) {
-			rc = EPKG_FATAL;
-		} else if (pkg_validate(pkg, j->db) == EPKG_OK) {
-			if (j->type == PKG_JOBS_UPGRADE && pkg_jobs_installed_local_pkg(j, pkg) != EPKG_OK) {
-				pkg_emit_error("%s is not installed, therefore upgrade is impossible",
-							pkg->name);
-				return (EPKG_NOTINSTALLED);
-			}
-			pkg->type = PKG_FILE;
-			pkg_jobs_add_req(j, pkg);
+	} else if (pkg_open(&pkg, jp->path, PKG_OPEN_MANIFEST_ONLY) != EPKG_OK) {
+		rc = EPKG_FATAL;
+	} else if (pkg_validate(pkg, j->db) == EPKG_OK) {
+		if (j->type == PKG_JOBS_UPGRADE &&
+		    _pkg_is_installed(j, pkg->name, MATCH_INTERNAL) != EPKG_OK) {
+			pkg_emit_error(
+			    "%s is not installed, therefore upgrade is impossible",
+			    pkg->name);
+			return (EPKG_NOTINSTALLED);
+		}
+		pkg->type = PKG_FILE;
+		pkg_jobs_add_req(j, pkg);
 
-			req = pkghash_get_value(j->request_add, pkg->uid);
-			if (req != NULL)
-				req->item->jp = jp;
-		}
-		else {
-			pkg_emit_error("cannot load %s: invalid format",
-					jp->pattern);
-			rc = EPKG_FATAL;
-		}
+		req = pkghash_get_value(j->request_add, pkg->uid);
+		if (req != NULL)
+			req->item->jp = jp;
+	} else {
+		pkg_emit_error("cannot load %s: invalid format", jp->pattern);
+		rc = EPKG_FATAL;
 	}
 
 	return (rc);
@@ -1087,7 +1072,13 @@ pkg_jobs_need_upgrade(charv_t *system_shlibs, struct pkg *rp, struct pkg *lp)
 		free(rp->reason);
 		xasprintf(&rp->reason, "ABI changed: '%s' -> '%s'",
 		    lp->abi, rp->abi);
-		assert(rp->reason != NULL);
+		return (true);
+	}
+
+	if (lp->vital != rp->vital) {
+		free(rp->reason);
+		xasprintf(&rp->reason, "Vital flag changed: '%s' -> '%s'",
+		    lp->vital ? "true" : "false", rp->vital ? "true" : "false");
 		return (true);
 	}
 
@@ -1477,8 +1468,6 @@ is_upgrade_candidate(struct pkg_jobs *j, struct pkg *pkg)
 				pkgdb_it_free(it);
 				return (true);
 			}
-			pkg_free(p);
-			p = NULL;
 		}
 		pkgdb_it_free(it);
 		return (false);
@@ -1501,8 +1490,6 @@ pkg_jobs_find_upgrade_candidates(struct pkg_jobs *j)
 	while (pkgdb_it_next(it, &pkg, PKG_LOAD_BASIC) == EPKG_OK) {
 		if (is_upgrade_candidate(j, pkg))
 			vec_push(candidates, pkg->id);
-		pkg_free(pkg);
-		pkg = NULL;
 	}
 	pkgdb_it_free(it);
 
@@ -1625,8 +1612,8 @@ jobs_solve_install_upgrade(struct pkg_jobs *j)
 	pkghash_it it;
 
 	/* Check for new pkg. Skip for 'upgrade -F'. */
-	if (((j->flags & PKG_FLAG_SKIP_INSTALL) == 0 &&
-	    (j->flags & PKG_FLAG_DRY_RUN) == 0) &&
+	if ((j->flags & PKG_FLAG_SKIP_INSTALL) == 0 &&
+	    (j->flags & PKG_FLAG_DRY_RUN) == 0 &&
 	    (j->flags & PKG_FLAG_PKG_VERSION_TEST) == PKG_FLAG_PKG_VERSION_TEST)
 		if (new_pkg_version(j)) {
 			j->flags &= ~PKG_FLAG_PKG_VERSION_TEST;
@@ -1826,7 +1813,6 @@ static int
 pkg_jobs_run_solver(struct pkg_jobs *j)
 {
 	int ret;
-	const char *cudf_solver;
 
 	pkgdb_begin_solver(j->db);
 
@@ -1849,9 +1835,10 @@ pkg_jobs_run_solver(struct pkg_jobs *j)
 		return (EPKG_FATAL);
 	}
 
-	cudf_solver = pkg_object_string(pkg_config_get("CUDF_SOLVER"));
-
 	if (ret == EPKG_OK) {
+		const char *cudf_solver;
+
+		cudf_solver = pkg_object_string(pkg_config_get("CUDF_SOLVER"));
 		if (cudf_solver != NULL) {
 			ret = solve_with_external_cudf_solver(j, cudf_solver);
 		} else {
