@@ -74,7 +74,7 @@ extern struct pkg_ctx ctx;
 */
 
 #define DB_SCHEMA_MAJOR	0
-#define DB_SCHEMA_MINOR	36
+#define DB_SCHEMA_MINOR	37
 
 #define DBVERSION (DB_SCHEMA_MAJOR * 1000 + DB_SCHEMA_MINOR)
 
@@ -416,12 +416,22 @@ pkgdb_init(sqlite3 *sdb)
 	"CREATE TABLE files ("
 		"path TEXT PRIMARY KEY,"
 		"sha256 TEXT,"
+		"uname TEXT,"
+		"gname TEXT,"
+		"perm INTEGER,"
+		"fflags INTEGER,"
+		"mtime INTEGER,"
+		"symlink_target TEXT,"
 		"package_id INTEGER REFERENCES packages(id) ON DELETE CASCADE"
 			" ON UPDATE CASCADE"
 	");"
 	"CREATE TABLE directories ("
 		"id INTEGER PRIMARY KEY,"
-		"path TEXT NOT NULL UNIQUE"
+		"path TEXT NOT NULL UNIQUE,"
+		"uname TEXT,"
+		"gname TEXT,"
+		"perm INTEGER,"
+		"fflags INTEGER"
 	");"
 	"CREATE TABLE pkg_directories ("
 		"package_id INTEGER REFERENCES packages(id) ON DELETE CASCADE"
@@ -1269,7 +1279,7 @@ pkgdb_transaction_rollback(struct pkgdb *db, const char *savepoint)
  * CASE_SENSITIVE_MATCH in pkg.conf and then possbily reset again in
  * pkg search et al according to command line flags */
 
-static bool _case_sensitive_flag = false;
+static bool _case_sensitive_flag = true;
 
 void
 pkgdb_set_case_sensitivity(bool case_sensitive)
@@ -1356,20 +1366,23 @@ static sql_prstmt sql_prepared_statements[PRSTMT_LAST] = {
 	},
 	[FILES] = {
 		NULL,
-		"INSERT INTO files (path, sha256, package_id) "
-		"VALUES (?1, ?2, ?3)",
-		"TTI",
+		"INSERT INTO files (path, sha256, uname, gname, "
+		"perm, fflags, symlink_target, mtime, package_id) "
+		"VALUES (?1, ?2, ?3, ?4, ?5, ?6, ?7, ?8, ?9)",
+		"TTTTIITII",
 	},
 	[FILES_REPLACE] = {
 		NULL,
-		"INSERT OR REPLACE INTO files (path, sha256, package_id) "
-		"VALUES (?1, ?2, ?3)",
-		"TTI",
+		"INSERT OR REPLACE INTO files (path, sha256, uname, gname, "
+		"perm, fflags, symlink_target, mtime, package_id) "
+		"VALUES (?1, ?2, ?3, ?4, ?5, ?6, ?7, ?8, ?9)",
+		"TTTTIITII",
 	},
 	[DIRS1] = {
 		NULL,
-		"INSERT OR IGNORE INTO directories(path) VALUES(?1)",
-		"T",
+		"INSERT OR IGNORE INTO directories(path, uname, gname, perm, fflags) "
+		"VALUES(?1,?2,?3,?4,?5)",
+		"TTTIII",
 	},
 	[DIRS2] = {
 		NULL,
@@ -1652,6 +1665,31 @@ prstmt_finalize(struct pkgdb *db)
 	return;
 }
 
+static const char *
+_pkgdb_empty_str_null(const char *str)
+{
+	return (str[0] == '\0' ? NULL : str);
+}
+
+static int
+run_pkg_prstmt(struct pkg *pkg)
+{
+	const char *arch;
+	char *msg;
+	int ret;
+
+	/* Prefer new ABI over old one */
+	arch = pkg->abi != NULL ? pkg->abi : pkg->altabi;
+	msg = pkg_message_to_str(pkg);
+	ret = run_prstmt(PKG, pkg->origin, pkg->name, pkg->version,
+	    pkg->comment, pkg->desc, msg, arch, pkg->maintainer,
+	    pkg->www, pkg->prefix, pkg->flatsize, (int64_t)pkg->automatic,
+	    (int64_t)pkg->licenselogic, pkg->digest, pkg->dep_formula,
+	    (int64_t)pkg->vital);
+	free(msg);
+	return (ret);
+}
+
 /*
  * Register a package in the database.  If successful, the caller is required to
  * call pkgdb_register_finale() in order to either commit or roll back the
@@ -1669,18 +1707,10 @@ pkgdb_register_pkg(struct pkgdb *db, struct pkg *pkg, int forced,
 	struct pkg_conflict	*conflict = NULL;
 	struct pkg_config_file	*cf = NULL;
 	struct pkgdb_it		*it = NULL;
-	char			*msg = NULL;
-
 	sqlite3			*s;
-
 	int			 ret;
-	int			 retcode = EPKG_FATAL;
 	int64_t			 package_id;
 
-	const char		*arch;
-
-	assert(db != NULL);
-
 	if (pkg_is_valid(pkg) != EPKG_OK) {
 		pkg_emit_error("the package is not valid");
 		return (EPKG_FATAL);
@@ -1691,17 +1721,10 @@ pkgdb_register_pkg(struct pkgdb *db, struct pkg *pkg, int forced,
 	if (pkgdb_transaction_begin_sqlite(s, savepoint) != EPKG_OK)
 		return (EPKG_FATAL);
 
-	/* Prefer new ABI over old one */
-	arch = pkg->abi != NULL ? pkg->abi : pkg->altabi;
-
 	/*
 	 * Insert package record
 	 */
-	msg = pkg_message_to_str(pkg);
-	ret = run_prstmt(PKG, pkg->origin, pkg->name, pkg->version,
-	    pkg->comment, pkg->desc, msg, arch, pkg->maintainer,
-	    pkg->www, pkg->prefix, pkg->flatsize, (int64_t)pkg->automatic,
-	    (int64_t)pkg->licenselogic, pkg->digest, pkg->dep_formula, (int64_t)pkg->vital);
+	ret = run_pkg_prstmt(pkg);
 	if (ret != SQLITE_DONE) {
 		ERROR_STMT_SQLITE(s, STMT(PKG));
 		goto cleanup;
@@ -1747,7 +1770,13 @@ pkgdb_register_pkg(struct pkgdb *db, struct pkg *pkg, int forced,
 			printf("matched\n");
 		}
 
-		ret = run_prstmt(FILES, file->path, file->sum, package_id);
+		ret = run_prstmt(FILES, file->path, file->sum,
+				 _pkgdb_empty_str_null(file->uname),
+				 _pkgdb_empty_str_null(file->gname),
+				 (int64_t)file->perm, (int64_t)file->fflags,
+				 _pkgdb_empty_str_null(file->symlink_target),
+				 (int64_t)file->time[1].tv_sec,
+				 package_id);
 		if (ret == SQLITE_DONE)
 			continue;
 		if (ret != SQLITE_CONSTRAINT) {
@@ -1765,6 +1794,11 @@ pkgdb_register_pkg(struct pkgdb *db, struct pkg *pkg, int forced,
 			/* Stray entry in the files table not related to
 			   any known package: overwrite this */
 			ret = run_prstmt(FILES_REPLACE, file->path, file->sum,
+					 _pkgdb_empty_str_null(file->uname),
+					 _pkgdb_empty_str_null(file->gname),
+					 (int64_t)file->perm, (int64_t)file->fflags,
+					 _pkgdb_empty_str_null(file->symlink_target),
+					 (int64_t)file->time[1].tv_sec,
 					 package_id);
 			pkgdb_it_free(it);
 			if (ret == SQLITE_DONE)
@@ -1822,7 +1856,10 @@ pkgdb_register_pkg(struct pkgdb *db, struct pkg *pkg, int forced,
 	 */
 
 	while (pkg_dirs(pkg, &dir) == EPKG_OK) {
-		if (run_prstmt(DIRS1, dir->path) != SQLITE_DONE) {
+		if (run_prstmt(DIRS1, dir->path, _pkgdb_empty_str_null(dir->uname),
+			       _pkgdb_empty_str_null(dir->gname),
+			       (int64_t)dir->perm, (int64_t)dir->fflags,
+			       dir->time[1].tv_sec) != SQLITE_DONE) {
 			ERROR_STMT_SQLITE(s, STMT(DIRS1));
 			goto cleanup;
 		}
@@ -1957,14 +1994,12 @@ pkgdb_register_pkg(struct pkgdb *db, struct pkg *pkg, int forced,
 	if (pkgdb_update_requires(pkg, package_id, s) != EPKG_OK)
 		goto cleanup;
 
-	retcode = EPKG_OK;
+	return (EPKG_OK);
 
 cleanup:
-	if (retcode != EPKG_OK)
-		(void)pkgdb_transaction_rollback_sqlite(s, savepoint);
-	free(msg);
+	(void)pkgdb_transaction_rollback_sqlite(s, savepoint);
 
-	return (retcode);
+	return (EPKG_FATAL);
 }
 
 static int
