@@ -211,9 +211,9 @@ pkgdb_load_deps(sqlite3 *sqlite, struct pkg *pkg)
 		"SELECT DISTINCT d.name, d.origin, p.version, 0"
 		"  FROM deps AS d"
 		"    LEFT JOIN packages AS p ON"
-		"    (p.origin = d.origin AND p.name = d.name)"
+		"    (p.name = d.name)"
 		"  WHERE d.package_id = ?1"
-		"  ORDER BY d.origin DESC";
+		"  ORDER BY d.name DESC";
 	const char formula_preamble[] = ""
 		"SELECT id,name,origin,version,locked FROM packages WHERE ";
 	const char options_sql[] = ""
@@ -388,7 +388,7 @@ pkgdb_load_files(sqlite3 *sqlite, struct pkg *pkg)
 	sqlite3_stmt	*stmt = NULL;
 	int		 ret;
 	const char	 sql[] = ""
-		"SELECT path, sha256"
+		"SELECT path, sha256, uname, gname, perm, fflags, mtime, symlink_target "
 		"  FROM files"
 		"  WHERE package_id = ?1"
 		"  ORDER BY PATH ASC";
@@ -412,8 +412,16 @@ pkgdb_load_files(sqlite3 *sqlite, struct pkg *pkg)
 	pkgdb_debug(4, stmt);
 
 	while (sqlite3_step(stmt) == SQLITE_ROW) {
-		pkg_addfile(pkg, sqlite3_column_text(stmt, 0),
-		    sqlite3_column_text(stmt, 1), false);
+		const char *path = sqlite3_column_text(stmt, 0);
+		const char *sum = sqlite3_column_text(stmt, 1);
+		const char *uname = sqlite3_column_text(stmt, 2);
+		const char *gname = sqlite3_column_text(stmt, 3);
+		mode_t perm = sqlite3_column_int64(stmt, 4);
+		u_long fflags = sqlite3_column_int64(stmt, 5);
+		time_t mtime = sqlite3_column_int64(stmt, 6);
+		const char *symlink_target = sqlite3_column_text(stmt, 7);
+		pkg_addfile_attr(pkg, path, sum, uname, gname, perm, fflags,
+				 mtime, symlink_target, false);
 	}
 	sqlite3_finalize(stmt);
 
@@ -445,7 +453,7 @@ static int
 pkgdb_load_dirs(sqlite3 *sqlite, struct pkg *pkg)
 {
 	const char	 sql[] = ""
-		"SELECT path, try"
+		"SELECT path, uname, gname, perm, fflags, try"
 		"  FROM pkg_directories, directories"
 		"  WHERE package_id = ?1"
 		"    AND directory_id = directories.id"
@@ -467,7 +475,12 @@ pkgdb_load_dirs(sqlite3 *sqlite, struct pkg *pkg)
 	pkgdb_debug(4, stmt);
 
 	while ((ret = sqlite3_step(stmt)) == SQLITE_ROW) {
-		pkg_adddir(pkg, sqlite3_column_text(stmt, 0), false);
+		const char *path = sqlite3_column_text(stmt, 0);
+		const char *uname = sqlite3_column_text(stmt, 1);
+		const char *gname = sqlite3_column_text(stmt, 2);
+		mode_t perm = sqlite3_column_int64(stmt, 3);
+		u_long fflags = sqlite3_column_int64(stmt, 4);
+		pkg_adddir_attr(pkg, path, uname, gname, perm, fflags, false);
 	}
 
 	if (ret != SQLITE_DONE) {
@@ -979,7 +992,7 @@ populate_pkg(sqlite3_stmt *stmt, struct pkg *pkg) {
 	pkg->altabi = xstrdup(legacyarch);
 }
 
-static struct load_on_flag {
+static const struct load_on_flag {
 	int	flag;
 	int	(*load)(sqlite3 *sqlite, struct pkg *p);
 } load_on_flag[] = {
@@ -1095,24 +1108,26 @@ pkgdb_sqlite_it_next(struct pkgdb_sqlite_it *it,
 int
 pkgdb_it_next(struct pkgdb_it *it, struct pkg **pkg_p, unsigned flags)
 {
-	struct pkg_repo_it *rit;
-	int ret = EPKG_END;
-
 	assert(it != NULL);
 
 	if (it->local != NULL && !it->local->finished) {
-		int r = pkgdb_sqlite_it_next(it->local, pkg_p, flags);
-		if ( r != EPKG_END)
-			return (r);
+		int ret;
+
+		ret = pkgdb_sqlite_it_next(it->local, pkg_p, flags);
+		if (ret != EPKG_END)
+			return (ret);
 	}
 
 	if (vec_len(&it->remote) != 0) {
+		struct pkg_repo_it *rit;
+		int ret;
+
 		if (it->remote_pos >= it->remote.len)
 			it->remote_pos = 0;
-		struct pkg_repo_it *rit = it->remote.d[it->remote_pos];
+		rit = it->remote.d[it->remote_pos];
 		ret = rit->ops->next(rit, pkg_p, flags);
 		if (ret != EPKG_OK) {
-			if (it->remote_pos == it->remote.len -1 )
+			if (it->remote_pos == it->remote.len - 1)
 				return (EPKG_END);
 			it->remote_pos++;
 			return (pkgdb_it_next(it, pkg_p, flags));
@@ -1124,7 +1139,7 @@ pkgdb_it_next(struct pkgdb_it *it, struct pkg **pkg_p, unsigned flags)
 		return (EPKG_OK);
 	}
 
-	return ret;
+	return (EPKG_END);
 }
 
 // TODO: Why doesn't this function handle remote?
