@@ -145,7 +145,6 @@ struct pkg_ctx {
 	bool backup_libraries;
 	const char *backup_library_path;
 	bool triggers;
-	const char *triggers_path;
 	pkghash *touched_dir_hash;
 	bool defer_triggers;
 	bool repo_accept_legacy_pkg;
@@ -378,6 +377,7 @@ struct pkg_file {
 	gid_t		 gid;
 	char		 temppath[MAXPATHLEN];
 	u_long		 fflags;
+	char		 symlink_target[MAXPATHLEN];
 	struct pkg_config_file *config;
 	struct timespec	 time[2];
 	struct pkg_file	*next, *prev;
@@ -468,7 +468,7 @@ struct pkg_repo_it_ops {
 
 struct pkg_repo_it {
 	struct pkg_repo *repo;
-	struct pkg_repo_it_ops *ops;
+	const struct pkg_repo_it_ops *ops;
 	int flags;
 	void *data;
 };
@@ -520,10 +520,10 @@ struct pkg_key {
 };
 
 struct pkg_repo {
-	struct pkg_repo_ops *ops;
+	const struct pkg_repo_ops *ops;
 
 	char *name;
-	struct fetcher *fetcher;
+	const struct fetcher *fetcher;
 	char *url;
 	char *pubkey;
 	mirror_t mirror_type;
@@ -792,7 +792,7 @@ int pkg_addfile(struct pkg *pkg, const char *path, const char *sha256,
     bool check_duplicates);
 int pkg_addfile_attr(struct pkg *pkg, const char *path, const char *sha256,
     const char *uname, const char *gname, mode_t perm, u_long fflags,
-    bool check_duplicates);
+    time_t mtime, const char *symlink_target, bool check_duplicates);
 
 int pkg_adddir(struct pkg *pkg, const char *path, bool check_duplicates);
 int pkg_adddir_attr(struct pkg *pkg, const char *path, const char *uname,
